==> 0.0.2.tar.gz: ios/Classes/ViewController.swift <==
//
//  ViewController.swift
//  StripeCustomDemo
//
//  Created by MacOs14 on 24/12/19.
//  Copyright © 2019 MacOs14. All rights reserved.
//

import UIKit
import Stripe

class ViewController: UIViewController {
    
//    var stripePublishableKey = "pk_test_0uVMBC6Im7TKjCuA47DP4dhn00ulPQWzXv"
//
//    var backendBaseURL: String? = "http://192.168.0.126:8000/api/v1/"
//
//    var appleMerchantID: String? = ""
    
    // These values will be shown to the user when they purchase with Apple Pay.
    let companyName = "Emoji Apparel"
    let paymentCurrency: String = "usd"
    
    var paymentContext: STPPaymentContext!
    
    var stripeKey : String
    var backendBaseURL : String
    var appleMerchantID : String
    var ephemeralKey : [AnyHashable : Any]
    var orderId : String
    var authToken: String
    var isFirstTime : Bool = true
    var onClose : (() -> ())?
        
    init(with stripeKey : String, backendBaseURL: String, appleMerchantID: String, ephemeralKey : [AnyHashable : Any], orderId: String, authToken: String) {
        self.stripeKey = stripeKey
        self.backendBaseURL = backendBaseURL
        self.appleMerchantID = appleMerchantID
        self.ephemeralKey = ephemeralKey
        self.orderId = orderId
        self.authToken = authToken
        
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        initTesttt()
    }
    
    func initTesttt() {
                
        assert(stripeKey.hasPrefix("pk_"), "You must set your Stripe publishable key at the top of CheckoutViewController.swift to run this app.")
        
        MyAPIClient.sharedClient.baseURLString = backendBaseURL
        MyAPIClient.sharedClient.ephemeralKey = ephemeralKey
        
        // This code is included here for the sake of readability, but in your application you should set up your configuration and theme earlier, preferably in your App Delegate.
        let config = STPPaymentConfiguration.shared()
        config.publishableKey = stripeKey
        config.appleMerchantIdentifier = appleMerchantID
        config.companyName = self.companyName
        
        let customerContext = STPCustomerContext(keyProvider: MyAPIClient.sharedClient)
        let paymentContext = STPPaymentContext(customerContext: customerContext,
                                               configuration: config,
                                               theme: STPTheme())
        let userInformation = STPUserInformation()
        paymentContext.prefilledInformation = userInformation
        paymentContext.paymentAmount = 100
        paymentContext.paymentCurrency = self.paymentCurrency
                
        self.paymentContext = paymentContext
        
        let numberFormatter = NumberFormatter()
        //        numberFormatter.locale = settings.currencyLocale
        numberFormatter.numberStyle = .currency
        numberFormatter.usesGroupingSeparator = true
        //        self.numberFormatter = numberFormatter
        
        self.paymentContext.delegate = self
        paymentContext.hostViewController = self
    }
    
    func tapPushPayment() {
        self.paymentContext.presentPaymentOptionsViewController()
    }
    
    func tapPushShipping() {
        self.paymentContext.pushShippingViewController()
    }
    
    @objc func didTapBuy() {
        //        self.paymentInProgress = true
        self.paymentContext.requestPayment()
    }
    
}

// MARK: STPPaymentContextDelegate
extension ViewController: STPPaymentContextDelegate {
    enum CheckoutError: Error {
        case unknown
        
        var localizedDescription: String {
            switch self {
            case .unknown:
                return "Unknown error"
            }
        }
    }
    func paymentContext(_ paymentContext: STPPaymentContext, didCreatePaymentResult paymentResult: STPPaymentResult, completion: @escaping STPPaymentStatusBlock) {
        // Request a PaymentIntent from your backend
        MyAPIClient.sharedClient.createPaymentIntent(order_id: self.orderId,authToken : self.authToken) { result in
            switch result {
            case .success(let clientSecret):
                // Assemble the PaymentIntent parameters
                let paymentIntentParams = STPPaymentIntentParams(clientSecret: clientSecret)
                paymentIntentParams.paymentMethodId = paymentResult.paymentMethod?.stripeId
                paymentIntentParams.configure(with: paymentResult)
                paymentIntentParams.savePaymentMethod = false
                
                // Confirm the PaymentIntent
                STPPaymentHandler.shared().confirmPayment(withParams: paymentIntentParams, authenticationContext: paymentContext) { status, paymentIntent, error in
                    switch status {
                    case .succeeded:
                        // Your backend asynchronously fulfills the customer's order, e.g. via webhook
                        completion(.success, nil)
                    case .failed:
                        completion(.error, error) // Report error
                    case .canceled:
                        completion(.userCancellation, nil) // Customer cancelled
                    @unknown default:
                        completion(.error, nil)
                    }
                }
            case .failure(let error):
                completion(.error, error) // Report error from your API
                break
            }
        }/*
        // Create the PaymentIntent on the backend
        // To speed this up, create the PaymentIntent earlier in the checkout flow and update it as necessary (e.g. when the cart subtotal updates or when shipping fees and taxes are calculated, instead of re-creating a PaymentIntent for every payment attempt.
        
        MyAPIClient.sharedClient.createPaymentIntent(order_id: self.orderId) { result in
            switch result {
            case .success(let clientSecret):
                // Confirm the PaymentIntent
                let paymentIntentParams = STPPaymentIntentParams(clientSecret: clientSecret)
                paymentIntentParams.configure(with: paymentResult)
                paymentIntentParams.returnURL = "payments-example://stripe-redirect"
//                paymentIntentParams.additionalAPIParameters = ["customer" : "cus_GR6znwnuGMGGqL"];
                STPPaymentHandler.shared().confirmPayment(withParams: paymentIntentParams, authenticationContext: paymentContext) { status, paymentIntent, error in
                    switch status {
                    case .succeeded:
                        // Our example backend asynchronously fulfills the customer's order via webhook
                        // See https://stripe.com/docs/payments/payment-intents/ios#fulfillment
                        completion(.success, nil)
                    case .failed:
                        completion(.error, error)
                    case .canceled:
                        completion(.userCancellation, nil)
                    @unknown default:
                        completion(.error, nil)
                    }
                }
            case .failure(let error):
                // A real app should retry this request if it was a network error.
                print("Failed to create a Payment Intent: \(error)")
                completion(.error, error)
                break
            }
        }*/
    }
    
    func paymentContext(_ paymentContext: STPPaymentContext, didFinishWith status: STPPaymentStatus, error: Error?) {
        self.onClose?()
        
        //        self.paymentInProgress = false
        let title: String
        let message: String
        switch status {
        case .error:
            title = "Error"
            message = error?.localizedDescription ?? ""
        case .success:
            title = "Success"
            message = "Your purchase was successful!"
        case .userCancellation:
            return()
        @unknown default:
            return()
        }
        let alertController = UIAlertController(title: title, message: message, preferredStyle: .alert)
        let action = UIAlertAction(title: "OK", style: .default, handler: nil)
        alertController.addAction(action)
        self.present(alertController, animated: true, completion: nil)
    }
    
    func paymentContextDidChange(_ paymentContext: STPPaymentContext) {
        
        print(paymentContext.loading)
        
        if let paymentOption = paymentContext.selectedPaymentOption {
            print(paymentOption.label)
            DispatchQueue.main.asyncAfter(deadline: .now() + 1.0) {
                if (self.isFirstTime) {
                    self.tapPushPayment()
//                    self.didTapBuy()
                    self.isFirstTime = false
                }
                else {
                    self.didTapBuy()
                }
            }
        } else {
            print("Select Payment")
        }
        
        /*self.paymentRow.loading = paymentContext.loading
         if let paymentOption = paymentContext.selectedPaymentOption {
         self.paymentRow.detail = paymentOption.label
         } else {
         self.paymentRow.detail = "Select Payment"
         }
         if let shippingMethod = paymentContext.selectedShippingMethod {
         self.shippingRow?.detail = shippingMethod.label
         } else {
         self.shippingRow?.detail = "Select address"
         }
         self.totalRow.detail = self.numberFormatter.string(from: NSNumber(value: Float(self.paymentContext.paymentAmount)/100))!*/
        
        //        buyButton.isEnabled = paymentContext.selectedPaymentOption != nil
        //            && (paymentContext.selectedShippingMethod != nil || self.shippingRow == nil)
        
        /*if (paymentContext.selectedPaymentOption != nil) {
            //            self.paymentInProgress = true
            self.paymentContext.requestPayment()
        }
        else {
            //            tapPushPayment()
        }*/
    }
    
    func paymentContext(_ paymentContext: STPPaymentContext, didFailToLoadWithError error: Error) {
        let alertController = UIAlertController(
            title: "Error",
            message: error.localizedDescription,
            preferredStyle: .alert
        )
        let cancel = UIAlertAction(title: "Cancel", style: .cancel, handler: { action in
            // Need to assign to _ because optional binding loses @discardableResult value
            // https://bugs.swift.org/browse/SR-1681
            self.onClose?()
            _ = self.navigationController?.popViewController(animated: true)
        })
        let retry = UIAlertAction(title: "Retry", style: .default, handler: { action in
            self.paymentContext.retryLoading()
        })
        alertController.addAction(cancel)
        alertController.addAction(retry)
        self.present(alertController, animated: true, completion: nil)
    }
    
    // Note: this delegate method is optional. If you do not need to collect a
    // shipping method from your user, you should not implement this method.
    func paymentContext(_ paymentContext: STPPaymentContext, didUpdateShippingAddress address: STPAddress, completion: @escaping STPShippingMethodsCompletionBlock) {
        let upsGround = PKShippingMethod()
        upsGround.amount = 0
        upsGround.label = "UPS Ground"
        upsGround.detail = "Arrives in 3-5 days"
        upsGround.identifier = "ups_ground"
        let upsWorldwide = PKShippingMethod()
        upsWorldwide.amount = 10.99
        upsWorldwide.label = "UPS Worldwide Express"
        upsWorldwide.detail = "Arrives in 1-3 days"
        upsWorldwide.identifier = "ups_worldwide"
        let fedEx = PKShippingMethod()
        fedEx.amount = 5.99
        fedEx.label = "FedEx"
        fedEx.detail = "Arrives tomorrow"
        fedEx.identifier = "fedex"
        
        DispatchQueue.main.asyncAfter(deadline: .now() + 1.0) {
            if address.country == nil || address.country == "US" {
                completion(.valid, nil, [upsGround, fedEx], fedEx)
            } else if address.country == "AQ" {
                let error = NSError(domain: "ShippingError", code: 123, userInfo: [NSLocalizedDescriptionKey: "Invalid Shipping Address",
                                                                                   NSLocalizedFailureReasonErrorKey: "We can't ship to this country."])
                completion(.invalid, error, nil, nil)
            } else {
                fedEx.amount = 20.99
                fedEx.identifier = "fedex_world"
                completion(.valid, nil, [upsWorldwide, fedEx], fedEx)
            }
        }
    }
}

==> 0.0.3.tar.gz: ios/Classes/ViewController.swift <==
//
//  ViewController.swift
//  StripeCustomDemo
//
//  Created by MacOs14 on 24/12/19.
//  Copyright © 2019 MacOs14. All rights reserved.
//

import UIKit
import Stripe

class ViewController: UIViewController {
    
//    var stripePublishableKey = "pk_test_0uVMBC6Im7TKjCuA47DP4dhn00ulPQWzXv"
//
//    var backendBaseURL: String? = "http://192.168.0.126:8000/api/v1/"
//
//    var appleMerchantID: String? = ""
    
    // These values will be shown to the user when they purchase with Apple Pay.
    let companyName = "Emoji Apparel"
    let paymentCurrency: String = "usd"
    
    var paymentContext: STPPaymentContext!
    
    var stripeKey : String
    var backendBaseURL : String
    var appleMerchantID : String
    var ephemeralKey : [AnyHashable : Any]
    var orderId : String
    var authToken: String
    var isFirstTime : Bool = true
    var onClose : (() -> ())!
    var onSuccess : ((_ stripeId: String) -> ())!
    var onError : ((_ errorMessage: String) -> ())!
    var onCancel : (() -> ())!
        
    init(with stripeKey : String, backendBaseURL: String, appleMerchantID: String, ephemeralKey : [AnyHashable : Any], orderId: String, authToken: String) {
        self.stripeKey = stripeKey
        self.backendBaseURL = backendBaseURL
        self.appleMerchantID = appleMerchantID
        self.ephemeralKey = ephemeralKey
        self.orderId = orderId
        self.authToken = authToken
        
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        initTesttt()
    }
    
    func initTesttt() {
                
        assert(stripeKey.hasPrefix("pk_"), "You must set your Stripe publishable key at the top of CheckoutViewController.swift to run this app.")
        
        MyAPIClient.sharedClient.baseURLString = backendBaseURL
        MyAPIClient.sharedClient.ephemeralKey = ephemeralKey
        
        // This code is included here for the sake of readability, but in your application you should set up your configuration and theme earlier, preferably in your App Delegate.
        let config = STPPaymentConfiguration.shared()
        config.publishableKey = stripeKey
        config.appleMerchantIdentifier = appleMerchantID
        config.companyName = self.companyName
        config.requiredBillingAddressFields = STPBillingAddressFields.name
        
        let customerContext = STPCustomerContext(keyProvider: MyAPIClient.sharedClient)
        let paymentContext = STPPaymentContext(customerContext: customerContext,
                                               configuration: config,
                                               theme: STPTheme())
        let userInformation = STPUserInformation()
        paymentContext.prefilledInformation = userInformation
        paymentContext.paymentAmount = 100
        paymentContext.paymentCurrency = self.paymentCurrency
                
        self.paymentContext = paymentContext
        
        let numberFormatter = NumberFormatter()
        //        numberFormatter.locale = settings.currencyLocale
        numberFormatter.numberStyle = .currency
        numberFormatter.usesGroupingSeparator = true
        //        self.numberFormatter = numberFormatter
        
        self.paymentContext.delegate = self
        paymentContext.hostViewController = self
    }
    
    func tapPushPayment() {
        self.paymentContext.presentPaymentOptionsViewController()
    }
    
    @objc func didTapBuy() {
        self.paymentContext.requestPayment()
    }
}

// MARK: STPPaymentContextDelegate
extension ViewController: STPPaymentContextDelegate {
    enum CheckoutError: Error {
        case unknown
        
        var localizedDescription: String {
            switch self {
            case .unknown:
                return "Unknown error"
            }
        }
    }
    func paymentContext(_ paymentContext: STPPaymentContext, didCreatePaymentResult paymentResult: STPPaymentResult, completion: @escaping STPPaymentStatusBlock) {
        // Request a PaymentIntent from your backend
        MyAPIClient.sharedClient.createPaymentIntent(order_id: self.orderId,authToken : self.authToken) { result in
            switch result {
            case .success(let clientSecret):
                // Assemble the PaymentIntent parameters
                let paymentIntentParams = STPPaymentIntentParams(clientSecret: clientSecret)
                paymentIntentParams.paymentMethodId = paymentResult.paymentMethod?.stripeId
                paymentIntentParams.configure(with: paymentResult)
//                paymentIntentParams.paymentMethodParams = paymentResult.paymentMethodParams
                paymentIntentParams.returnURL = "stripedemo://stripe-redirect"
                
                // Confirm the PaymentIntent
                STPPaymentHandler.shared().confirmPayment(withParams: paymentIntentParams, authenticationContext: self.paymentContext) { status, paymentIntent, error in
                    switch status {
                    case .succeeded:
                        // Your backend asynchronously fulfills the customer's order, e.g. via webhook
                        completion(.success, nil)
                    case .failed:
                        completion(.error, error) // Report error
                    case .canceled:
                        completion(.userCancellation, nil) // Customer cancelled
                    @unknown default:
                        completion(.error, nil)
                    }
                }
            case .failure(let error):
                completion(.error, error) // Report error from your API
                break
            }
        }
        /*
        // Create the PaymentIntent on the backend
        // To speed this up, create the PaymentIntent earlier in the checkout flow and update it as necessary (e.g. when the cart subtotal updates or when shipping fees and taxes are calculated, instead of re-creating a PaymentIntent for every payment attempt.
        
        MyAPIClient.sharedClient.createPaymentIntent(order_id: self.orderId) { result in
            switch result {
            case .success(let clientSecret):
                // Confirm the PaymentIntent
                let paymentIntentParams = STPPaymentIntentParams(clientSecret: clientSecret)
                paymentIntentParams.configure(with: paymentResult)
                paymentIntentParams.returnURL = "payments-example://stripe-redirect"
//                paymentIntentParams.additionalAPIParameters = ["customer" : "cus_GR6znwnuGMGGqL"];
                STPPaymentHandler.shared().confirmPayment(withParams: paymentIntentParams, authenticationContext: paymentContext) { status, paymentIntent, error in
                    switch status {
                    case .succeeded:
                        // Our example backend asynchronously fulfills the customer's order via webhook
                        // See https://stripe.com/docs/payments/payment-intents/ios#fulfillment
                        completion(.success, nil)
                    case .failed:
                        completion(.error, error)
                    case .canceled:
                        completion(.userCancellation, nil)
                    @unknown default:
                        completion(.error, nil)
                    }
                }
            case .failure(let error):
                // A real app should retry this request if it was a network error.
                print("Failed to create a Payment Intent: \(error)")
                completion(.error, error)
                break
            }
        }*/
    }
    
    func paymentContext(_ paymentContext: STPPaymentContext, didFinishWith status: STPPaymentStatus, error: Error?) {
        
        switch status {
        case .error:
            self.onError(error?.localizedDescription ?? "")
        case .success:
            self.onSuccess("tset")
        case .userCancellation:
            self.onCancel()
        @unknown default:
            self.onClose()
            break
        }
        
        self.navigationController?.popViewController(animated: true)
    }
    
    func paymentContextDidChange(_ paymentContext: STPPaymentContext) {
        
        print(paymentContext.loading)
        
        if let paymentOption = paymentContext.selectedPaymentOption {
            print(paymentOption.label)
            DispatchQueue.main.asyncAfter(deadline: .now() + 1.0) {
                if (self.isFirstTime) {
                    self.tapPushPayment()
                    self.isFirstTime = false
                }
                else {
                    self.didTapBuy()
                }
            }
        } else {
            print("Select Payment")
        }
        
        /*self.paymentRow.loading = paymentContext.loading
         if let paymentOption = paymentContext.selectedPaymentOption {
         self.paymentRow.detail = paymentOption.label
         } else {
         self.paymentRow.detail = "Select Payment"
         }
         if let shippingMethod = paymentContext.selectedShippingMethod {
         self.shippingRow?.detail = shippingMethod.label
         } else {
         self.shippingRow?.detail = "Select address"
         }
         self.totalRow.detail = self.numberFormatter.string(from: NSNumber(value: Float(self.paymentContext.paymentAmount)/100))!*/
        
        //        buyButton.isEnabled = paymentContext.selectedPaymentOption != nil
        //            && (paymentContext.selectedShippingMethod != nil || self.shippingRow == nil)
        
        /*if (paymentContext.selectedPaymentOption != nil) {
            //            self.paymentInProgress = true
            self.paymentContext.requestPayment()
        }
        else {
            //            tapPushPayment()
        }*/
    }
    
    func paymentContext(_ paymentContext: STPPaymentContext, didFailToLoadWithError error: Error) {
        let alertController = UIAlertController(
            title: "Error",
            message: error.localizedDescription,
            preferredStyle: .alert
        )
        let cancel = UIAlertAction(title: "Cancel", style: .cancel, handler: { action in
            // Need to assign to _ because optional binding loses @discardableResult value
            // https://bugs.swift.org/browse/SR-1681
            self.onClose()
            self.navigationController?.popViewController(animated: true)
        })
        let retry = UIAlertAction(title: "Retry", style: .default, handler: { action in
            self.paymentContext.retryLoading()
        })
        alertController.addAction(cancel)
        alertController.addAction(retry)
        self.present(alertController, animated: true, completion: nil)
    }
    
    // Note: this delegate method is optional. If you do not need to collect a
    // shipping method from your user, you should not implement this method.
    func paymentContext(_ paymentContext: STPPaymentContext, didUpdateShippingAddress address: STPAddress, completion: @escaping STPShippingMethodsCompletionBlock) {
        let upsGround = PKShippingMethod()
        upsGround.amount = 0
        upsGround.label = "UPS Ground"
        upsGround.detail = "Arrives in 3-5 days"
        upsGround.identifier = "ups_ground"
        let upsWorldwide = PKShippingMethod()
        upsWorldwide.amount = 10.99
        upsWorldwide.label = "UPS Worldwide Express"
        upsWorldwide.detail = "Arrives in 1-3 days"
        upsWorldwide.identifier = "ups_worldwide"
        let fedEx = PKShippingMethod()
        fedEx.amount = 5.99
        fedEx.label = "FedEx"
        fedEx.detail = "Arrives tomorrow"
        fedEx.identifier = "fedex"
        
        DispatchQueue.main.asyncAfter(deadline: .now() + 1.0) {
            if address.country == nil || address.country == "US" {
                completion(.valid, nil, [upsGround, fedEx], fedEx)
            } else if address.country == "AQ" {
                let error = NSError(domain: "ShippingError", code: 123, userInfo: [NSLocalizedDescriptionKey: "Invalid Shipping Address",
                                                                                   NSLocalizedFailureReasonErrorKey: "We can't ship to this country."])
                completion(.invalid, error, nil, nil)
            } else {
                fedEx.amount = 20.99
                fedEx.identifier = "fedex_world"
                completion(.valid, nil, [upsWorldwide, fedEx], fedEx)
            }
        }
    }
}
